@@ -23,6 +23,7 @@
 
 #include <bson.h>
 
+#include "mongoc-macros.h"
 
 BSON_BEGIN_DECLS
 
@@ -30,13 +31,13 @@ BSON_BEGIN_DECLS
 typedef struct _mongoc_matcher_t mongoc_matcher_t;
 
 
-BSON_EXPORT (mongoc_matcher_t *)
+MONGOC_EXPORT (mongoc_matcher_t *)
 mongoc_matcher_new (const bson_t *query,
                     bson_error_t *error) BSON_GNUC_DEPRECATED;
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_matcher_match (const mongoc_matcher_t *matcher,
                       const bson_t *document) BSON_GNUC_DEPRECATED;
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_matcher_destroy (mongoc_matcher_t *matcher) BSON_GNUC_DEPRECATED;
 
 
